@@ -1,52 +1,54 @@
-/* -*- c++ -*- (enables emacs c++ mode)                                    */
-/* *********************************************************************** */
-/*                                                                         */
-/* Library : Getfem Interface (getfemint)                                  */
-/* File    : getfemint.h : global definitions for Getfem interface.        */
-/*     									   */
-/*                                                                         */
-/* Date : September 26, 2001.                                              */
-/* Author : Yves Renard, Yves.Renard@gmm.insa-tlse.fr                      */
-/*                                                                         */
-/* *********************************************************************** */
-/*                                                                         */
-/* Copyright (C) 2001  Yves Renard.                                        */
-/*                                                                         */
-/* This file is a part of GETFEM++                                         */
-/*                                                                         */
-/* This program is free software; you can redistribute it and/or modify    */
-/* it under the terms of the GNU General Public License as published by    */
-/* the Free Software Foundation; version 2 of the License.                 */
-/*                                                                         */
-/* This program is distributed in the hope that it will be useful,         */
-/* but WITHOUT ANY WARRANTY; without even the implied warranty of          */
-/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
-/* GNU General Public License for more details.                            */
-/*                                                                         */
-/* You should have received a copy of the GNU General Public License       */
-/* along with this program; if not, write to the Free Software Foundation, */
-/* Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.         */
-/*                                                                         */
-/* *********************************************************************** */
+// -*- c++ -*- (enables emacs c++ mode)
+//========================================================================
+//
+// Copyright (C) 2001-2006 Y. Renard, J. Pommier.
+//
+// This file is a part of GETFEM++
+//
+// Getfem++ is free software; you can redistribute it and/or modify
+// it under the terms of the GNU Lesser General Public License as
+// published by the Free Software Foundation; version 2.1 of the License.
+//
+// This program is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU Lesser General Public License for more details.
+// You should have received a copy of the GNU Lesser General Public
+// License along with this program; if not, write to the Free Software
+// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301,
+// USA.
+//
+//========================================================================
 
+/**\file getfemint.h
+   \brief main getfem-interface header.
+*/
 
 #ifndef GETFEMINT_H__
 #define GETFEMINT_H__
 
+/***************************************
+ * Interface a new class:
+ *    - add it to enum getfemint_class_id in gfi_array.h
+ *    - add it to name_of_getfemint_class_id etc in getfemint.cc
+ *    - add it in the list at the bottom of getfem.base.py
+ */
+
+
 #include <getfemint_std.h>
 #include <set>
 //#include <getfem_config.h>
-//#include <getfem_mesh.h>
+//#include <mesh.h>
 #include <getfem_mesh_fem.h>
 #include <gfi_array.h>
 #include <dal_shared_ptr.h>
+#include <getfem_mat_elem_type.h>
 
 namespace getfem {
   class stored_mesh_slice;
-  class getfem_mesh;
+  class mesh;
   class mesh_fem;
   class mat_elem_type;
-  typedef const mat_elem_type* pmat_elem_type; 
 }
 
 namespace getfemint
@@ -81,6 +83,9 @@ namespace getfemint
 
   class getfemint_mesh;
   class getfemint_mesh_fem;
+  class getfemint_mesh_im;
+  class getfemint_mdbrick;
+  class getfemint_mdstate;
   class getfemint_mesh_slice;
   class getfemint_precond;
   class getfemint_gsparse;
@@ -115,7 +120,10 @@ namespace getfemint
     array_dimensions(const gfi_array *mx) { 
       assign_dimensions(mx);
     }
-    array_dimensions(unsigned sz_) { sz = sz_; ndim_=1; sizes_[0]=sz_; }
+    explicit array_dimensions(unsigned sz_) { sz = sz_; ndim_=1; sizes_[0]=sz_; }
+    template <typename IVECT> void assign(const IVECT &v) {
+      for (unsigned i=0; i < v.size(); ++i) push_back(v[i]);
+    }
     unsigned size() const { return sz; }
     unsigned ndim() const { return ndim_; }
     /* dim(0) = first dimension, dim(-1) = last dimension, ..*/
@@ -123,9 +131,16 @@ namespace getfemint
     unsigned getm() const { return dim(0); }
     unsigned getn() const { return dim(1); }
     unsigned getp() const { return dim(2); }
+    unsigned getq() const { return dim(3); }
     const unsigned *sizes() const { return sizes_; }
     void reshape(unsigned n_, unsigned m_, unsigned p_=1);
     void opt_transform_col_vect_into_row_vect();
+    friend std::ostream& operator << (std::ostream &os, 
+				      const array_dimensions &d) {
+      os << d.dim(0);
+      for (unsigned i=1; i < d.ndim(); ++i) os << "x" << d.dim(i);
+      return os;
+    }
   };
 
 
@@ -240,26 +255,35 @@ namespace getfemint
     darray &real() { if (v != REAL) THROW_INTERNAL_ERROR; return *d; }
     carray &cplx() { if (v != COMPLEX) THROW_INTERNAL_ERROR; return *c; }
     
+    rcarray() : mx(0), d(0), c(0), v(REAL) {}
     rcarray(const gfi_array *mx_, int v_ = -1) : d(0), c(0) { assign(mx_,v_); }
     ~rcarray() { clear(); }    
     void assign(const gfi_array *mx_, int v_ = -1) {
       mx = mx_;
       v = v_; if (v == -1) v = gfi_array_is_complex(mx) ? COMPLEX : REAL;
       clear();
-      if (v == REAL) d = new darray(mx); else c = new carray(mx);
+      if (v == REAL) 
+	d.reset(new darray(mx)); 
+      else c.reset(new carray(mx));
     }
     bool is_complex() const { return v == COMPLEX; }
-    void to_complex() {
+    carray &to_complex() {
       if (v == REAL) 
-	{ c = new carray(mx); delete d; v = COMPLEX; }
+	{ c.reset(new carray(mx)); d.reset(0); v = COMPLEX; }
+      return cplx();
     }
-    void clear() { if (d) delete d; d = 0; if (c) delete c; c = 0; }
-    array_dimensions& sizes() { if (d) return *d; else return *c; }
-    const array_dimensions& sizes() const { if (d) return *d; else return *c; }
+    darray &to_real() {
+      if (v == COMPLEX) 
+	THROW_BADARG("expected a real (not a complex) array in this context");
+      return real();
+    }
+    void clear() { d.reset(0); c.reset(0); }
+    array_dimensions& sizes() { if (d.get()) return *d; else return *c; }
+    const array_dimensions& sizes() const { if (d.get()) return *d; else return *c; }
   private:
     const gfi_array *mx;
-    darray *d;
-    carray *c;
+    dal::shared_ptr<darray> d;
+    dal::shared_ptr<carray> c;
     int v;
   };
     
@@ -326,7 +350,9 @@ namespace gmm {
 
 namespace getfemint {
   inline bool is_static_object_cid(id_type cid) {
-    return (cid != MESHFEM_CLASS_ID && cid != MESH_CLASS_ID &&
+    return (cid != MESHFEM_CLASS_ID && cid != MESHIM_CLASS_ID &&
+	    cid != MESH_CLASS_ID && 
+	    cid != MDBRICK_CLASS_ID && cid != MDSTATE_CLASS_ID && 
 	    cid != SLICE_CLASS_ID && cid != POLY_CLASS_ID && 
             cid != PRECOND_CLASS_ID && cid != GSPARSE_CLASS_ID);
   }
@@ -358,35 +384,45 @@ namespace getfemint {
     bool                        is_object_id(id_type *pid=0, id_type *pcid=0);
     bool                        is_mesh();
     bool                        is_mesh_fem();
+    bool                        is_mesh_im();
+    bool                        is_mdbrick();
+    bool                        is_mdstate();
     bool                        is_mesh_slice();
     bool                        is_sparse() { return (gfi_array_get_class(arg) == GFI_SPARSE || is_gsparse()); };
     bool                        is_gsparse();
     bool                        is_complex(); /* true for complex garrays AND complex sparse matrices (native or gsparse) */
+    bool                        is_integer();
     int                         to_integer(int min_val=INT_MIN, int max_val=INT_MAX);
-    size_type                   to_convex_number(const getfem::getfem_mesh &m);
+    size_type                   to_convex_number(const getfem::mesh &m);
     double                      to_scalar(double min_val=-1e300, double max_val=1e300);
     complex_type                to_scalar(complex_type);
     std::string                 to_string();
     id_type                     to_object_id(id_type *pid=0, id_type *pcid=0);
     bgeot::base_poly *          to_poly();
     getfem::mesh_fem *          to_mesh_fem();
-    const getfem::getfem_mesh * to_const_mesh();
-    const getfem::getfem_mesh * to_const_mesh(id_type& id);
+    getfem::mesh_im *           to_mesh_im();
+    const getfem::mesh * to_const_mesh();
+    const getfem::mesh * to_const_mesh(id_type& id);
     getfemint_mesh *            to_getfemint_mesh();
     getfemint_mesh_fem *        to_getfemint_mesh_fem();
-    getfem::getfem_mesh *       to_mesh();
+    getfemint_mesh_im *         to_getfemint_mesh_im();
+    getfemint_mdbrick *         to_getfemint_mdbrick();
+    getfemint_mdstate *         to_getfemint_mdstate();
+    getfem::mesh *       to_mesh();
     getfemint_mesh_slice *      to_getfemint_mesh_slice();
     getfem::stored_mesh_slice * to_mesh_slice();
-    getfem::pintegration_method to_fem_interpolation();
+    getfem::pintegration_method to_integration_method();
     getfem::pfem                to_fem();
     getfem::pmat_elem_type      to_mat_elem_type();
     bgeot::pgeometric_trans     to_pgt();
     bgeot::pconvex_structure    to_convex_structure();
     getfemint_precond *         to_precond();
+    getfem::mesh_region         to_mesh_region();
 
     carray                      to_carray();
     carray                      to_carray(int expected_dim);
-    carray                      to_carray(int expected_n, int expected_m, int expected_k=1);
+    carray                      to_carray(int expected_n, int expected_m, 
+					  int expected_k=1, int expected_q=1);
 
     /* do not perform any check on the number of dimensions of the array */
     darray                      to_darray();
@@ -398,16 +434,19 @@ namespace getfemint {
        if any of the arguments has a value of -1, the corresponding dimension
        is not checked
      */
-    darray                      to_darray(int expected_m, int expected_n, int expected_k=1);
+    darray                      to_darray(int expected_m, int expected_n, 
+					  int expected_k=1, int expected_q=1);
 
     /* convertion to a real or complex array */
     rcarray                     to_rcarray();
     rcarray                     to_rcarray(int expected_dim);
-    rcarray                     to_rcarray(int expected_m, int expected_n, int expected_k=1);    
+    rcarray                     to_rcarray(int expected_m, int expected_n, 
+					   int expected_k=1, int expected_q=1);    
 
     iarray                      to_iarray();
     iarray                      to_iarray(int expected_dim);
-    iarray                      to_iarray(int expected_m, int expected_n, int expected_k=1);
+    iarray                      to_iarray(int expected_m, int expected_n, 
+					  int expected_k=1, int expected_q=1);
 
     /* template friendly version */
     garray<double>              to_garray(double) { return to_darray(); }
@@ -427,7 +466,7 @@ namespace getfemint {
     getfemint_gsparse *         to_getfemint_gsparse();
 
     mexarg_in &check_trailing_dimension(int expected_dim);
-    void check_dimensions(array_dimensions &v, int expected_m, int expected_n, int expected_p=-1);
+    void check_dimensions(array_dimensions &v, int expected_m, int expected_n, int expected_p=-1, int expected_q=-1);
     void check_dimensions(const array_dimensions &v, int expected_dim);
 private:
     friend class mexargs_in;
@@ -449,7 +488,7 @@ private:
     void from_string(const char *s);
     template<class STR_CONT> void from_string_container(const STR_CONT& s);
     void from_bit_vector(const dal::bit_vector& bv, int shift=config::base_index());
-
+    void from_mesh_region(const getfem::mesh_region &region);
     typedef enum { USE_NATIVE_SPARSE, USE_GSPARSE, USE_DEFAULT_SPARSE } output_sparse_fmt;
 
 
@@ -616,6 +655,7 @@ private:
     THROW_BADARG("Bad command name: " << cmd); }
 
   void check_cv_fem(const getfem::mesh_fem& mf, size_type cv);
+  void check_cv_im(const getfem::mesh_im& mim, size_type cv);
   
   const double& get_NaN();
   bool is_NaN(const double&);
